{% extends "accounts/accounts_base.html" %}

{% block title %}
Internal Error - Seattle Clearinghouse
{% endblock title %}

{% block content %}
	<div id="main">
		<div id="middle">

			<h2>Whoops, an internal error occurred.</h2>
			
			<p>We've been notified and are working on this. If this problem continues, please contact us.</p>
			
		</div>
	</div>
{% endblock content %}
