
/*******************************
* STYLE FOR DOWNLOAD PAGES
*******************************/

#downloadlink {
    margin: 0 auto 10px;
    width: 90%;
    text-align: center;
}

#downloadlink p {
    margin: 8px;
    color: #333;
}

#downloadlink a {
    padding: 5px;
    background-color: #fff;
}

table#downloads {
    margin: 10px auto 80px auto;
	border-spacing: 0;
	border: 1px solid #999;
	width: 90%;
	background-color: #fff;
	border-collapse: collapse;
}

table#downloads tr {
    border: 1px solid #ccc;
}

table#downloads tr td {
    padding: 20px 25px;
    font-size: 11pt;
}

table#downloads tr td a {
    color: #0066ff;
    font-size: 14pt;
    font-weight: bold;
}
