/*****************************************************************
<File Name>
	style.css

<Started>
	January 18, 2009

<Author>
	shawiz@cs.washington.edu
	Xuanhua (Sean) Ren

<Purpose>
	Provides the stylesheet for Seattle Clearinghouse


*****************************************************************/



/*******************
* RESET
********************/

* {
	margin: 0;
	padding: 0;
}

fieldset, img {
	border: 0;
}

ol {
    list-style-position: inside;
}

a {
    text-decoration: none;
    color: #000;    
}



/***********************
* GENERAL
************************/


body {
	background-color: #fff;
	font-family: "Segoe UI", "Lucida Grande", "Helvetica" , sans-serif;
	font-size: 12pt;
}

#header {
    width: 70%;
    margin: 30px auto 20px auto;
}

#header #logo {
    font-size: 36pt;
    font-weight: bold;
}

#header #caption {
    margin-left: 20px;
    font-size: 10pt;
}

#header #caption a {
	color: #0066FF;
}

#navigator {
    width: 100%;
    border-bottom: 3px solid #222;
}

#tabs {
    width: 70%;
    margin: 0 auto;
}

#meta {
    float: right;
    font-size: 13pt;
    margin: 5px 0;
}

#tabs table {
    border-collapse: collapse;
    border-spacing: 0;
}

#tabs table tr td {
   padding: 5px 15px;
   font-size: 14pt;
   border-top: 1px solid #222;
   border-right: 2px solid #222;
   border-left: 2px solid #222;
}

#tabs table tr td.active {
    background-color: #222;
    color: #fff;
}

#main {
    width: 100%;
    height: 400px;
    padding: 40px 0;
    background-image: url(../images/bg.png);
    background-repeat: repeat-x;
}

#middle {
    margin: 0 auto;
    width: 70%;
	padding-bottom: 60px;
}

.warning {
	border: 1px solid red;
	margin: 20px 60px;
	padding: 10px 20px;
	color: red;
	font-weight: bold;
	background-color: #f2dbdb;
	text-align: center;
}

.message {
	border: 1px solid green;
	margin: 20px 60px;
	padding: 10px 20px;
	color: green;
	font-weight: bold;
	background-color: #CCFFCC;
	text-align: center;
}

.tooltip {
	background-color: #efefef;
	color: #333;
	font-size: 10pt;
	padding: 3px;
    border: solid 1px #333;
	z-index:100;
}

ul.errorlist li {
	color: red;
	font-weight: normal;
	font-size: 10pt;
	background-color: #f2dbdb;
    padding: 3px;
    list-style-position: inside;
}


/******************************
* STYLESHEET FOR login.html and register.html
*******************************/
table.login{
    margin: 20px auto;
    border-collapse: collapse;
	border-spacing: 0;
	width: 70%;
	font-size: 14pt;
}

table.login tr td {
    height: 50px;
}

table.login tr td.key {
    text-align: right;
    height: 40px;
    font-weight: bold;
    width: 160px;
}

table.login tr td.notice {
    height: 40px;
}

table.login tr td.value {
    padding-left: 15px;
}

table.login tr td.value input[type="text"],
table.login tr td.value input[type="password"] {
    width: 180px;
}

table.login tr td.value input {
    padding: 1px;
    font-size: 14pt;
}

table.login tr td.value select {
    font-size: 10pt;
}


/******************************
* STYLESHEET FOR index.html
*******************************/

table.profile {
    margin-top: 20px;
    border-collapse: collapse;
	border-spacing: 0;
	width: 70%;
}

table.profile tr td.key {
    text-align: right;
    height: 30px;
    width: 150px;
    font-weight: bold;
}

table.profile tr td.value {
    padding-left: 15px;
    
}

table.profile tr.odd {
    background-color: #fff;
}

table.profile tr.even {
    background-color: #eee;
}

/*

#middle {
	float: right;
	width: 100px;
	border: 2px solid #666;
	background-color: #fff;
	padding: 10px;
	margin-top: 20px;
	margin-right: 60px;
	text-align: center;
	font-weight: bold;
}

*/

#download {
    float: right;
    border: 2px solid #666;
    background-color: #fff;
    padding: 10px;
    margin-top: 20px;
    margin-right: 0px;
    text-align: center;
    font-weight: bold;
}


/******************************
* STYLESHEET FOR mygeni.html
*******************************/

.bar {
    width: 100%;
    padding: 20px 0px;
    font-size: 11pt;
    color: #333;
}

.bar h3 {
    margin-bottom: 10px;
}

.vesselstatus {
	margin-top: 3px;
	float: right;
}

table.credits {
	border-spacing: 3px;
	border: 2px solid #999;
	width: 100%;
	height: 80px;
}

table.usage {
    border-spacing: 3px;
	border: 2px solid #999;
	width: 100%;
	height: 80px;
}

table.credits tr td, table.usage tr td {
	color: #666;
	text-align: center;
	padding: 8px;
}

table.credits tr td span, table.usage tr td span {
    font-size: 14pt;
    font-weight: bold;
}

table.usage tr td a {
	float: right;
	font-size: 12pt;
	color: #888;
	margin-left: 8px;
}

table.names {
	border-spacing: 0px;
	width: 100%;
}

table.names tr td {
    padding: 3px;
    text-align: center;
}

table.usage tr.theader {
    height: 15px;
    font-size: 10pt;
    font-weight: normal;
    text-align: right;
    padding-right: 10px;
}

table.users {
    display: none;
    width: 100%;
    margin: 20px auto;
    border-spacing: 3px;
    border: 2px solid #999;
}

table.users caption {
    font-weight: bold;
	padding: 20px auto 10px auto;
}

table.users tr td{
    height: 25px;
    text-align: center;
}
    
table.users tr th {
    font-weight: bold;
    background-color: #FFCCFF;
    color: #333;
}

table.users tr.odd {
    background-color: #fff;
}

table.users tr.even {
    background-color: #eee;
}

#overlay {
    position: absolute;
    top: 0;
    left: 0;
    width: 100%;
    height: 100%;
    background-color: #ccc;
    opacity: 0.3;
    display: none;
}

#dialogframe {
    position: absolute;
    top: 0;
    left: 0;
    width: 100%;
    display: none;
}

#dialogframe #getresourcesdialog,
#dialogframe #shareresourcesdialog {
    margin: 180px auto;
    padding: 20px;
    width: 300px;
    /* height: 140px; */
    background-color: #fff;
    border: 1px solid #999;
    text-align: center;
    display: none;
}

#dialogframe #getresourcesdialog h3,
#dialogframe #shareresourcesdialog h3,
#dialogframe #changepercentdialog h3 {
    margin: 5px auto;
    font-size: 16pt;
}

#dialogframe #getresourcesdialog form,
#dialogframe #shareresourcesdialog form {
    margin-top: 15px;
}

#dialogframe #getresourcesdialog form select {
    margin-top: 5px;
}

#dialogframe #getresourcesdialog input,
#dialogframe #getresourcesdialog button,
#dialogframe #shareresourcesdialog button,
#dialogframe #changepercentdialog button {
    margin-top: 20px;
    font-size: 10pt;
    padding: 1px;
}

#dialogframe #shareresourcesdialog span {
    display: block;
    width: 80px;
    text-align: right;
    float: left;
    margin-top: 5px;
}

#dialogframe #shareresourcesdialog input,
#dialogframe #changepercentdialog input {
    font-size: 10pt;
    margin-top: 5px;
    padding: 1px;
}


#dialogframe #changepercentdialog {
    margin: 180px auto;
    padding: 20px;
    width: 250px;
    /* height: 110px; */
    background-color: #fff;
    border: 1px solid #999;
    text-align: center;
    display: none;
}

#dialogframe .warning {
	color: red;
	font-weight: bold;
}


/******************************
* STYLESHEET FOR myvessels.html
*******************************/

#getresources {
    width: 90%;
    border: 2px solid #999;
    margin: 0 auto;
    background-color: #fff;
    padding: 10px 0;
    font-size: 12pt;
    text-align: center;
    color: #666;
}

#getresources #caption {
    color: #444;
    font-weight: bold;
    margin: 0 15px 0 0;
}

#getresources form input, #getresources form select {
    margin-right: 10px;
}

table.vessels {
    width: 90%;
    margin: 20px auto;
    border-spacing: 3px;
    border: 2px solid #999;
}

table.vessels tr td{
    height: 30px;
    text-align: center;
}
    
table.vessels tr th {
    font-weight: bold;
    background-color: #666;
    color: #fff;
}

table.vessels tr.odd {
    background-color: #fff;
}

table.vessels tr.even {
    background-color: #eee;
}

/******************************
* STYLESHEET FOR getdonations.html
*******************************/
#downloadlink {
    margin: 0 auto 10px;
    width: 90%;
    text-align: center;
}

#downloadlink p {
    margin: 8px;
    color: #333;
}

#downloadlink a {
    color: #0066ff;
    padding: 5px;
    background-color: #fff;
}

table#downloads {
    margin: 10px auto 80px auto;
	border-spacing: 0;
	border: 1px solid #999;
	width: 90%;
	background-color: #fff;
	border-collapse: collapse;
}

table#downloads tr {
    border: 1px solid #ccc;
}

table#downloads tr td {
    padding: 20px 25px;
    font-size: 11pt;
}

table#downloads tr td a {
    color: #0066ff;
    font-size: 14pt;
    font-weight: bold;
}


