"""
This is the configuration file used by the keydb api. Any program that uses
the keydb must have access to this file. Any program that shouldn't have
access to the keydb should not have access to this file.
"""

# Fill these in with the information needed to use the key database.
dbuser = ""
dbpass = ""
dbname = "keydb"
dbhost = "localhost"
