{% extends "control/control_base.html" %}

{% block title %}
Under Construction - Seattle Clearinghouse
{% endblock title %}

{% block content %}
<p align="center">
<h1>{{ msg }}</h1>
</p>
{% endblock content %}

