{% extends "common/geni_base.html" %}

{% block navigation %}
	<div id="navigator">
		<div id="tabs">
			<table>
				<tr>
				  {% block nav_getdonations %}
				  <td><a href="{% url installers username %}">Donate Resources</a></td>
				  {% endblock nav_getdonations %}

				  {% block nav_help %}
				  <td><a href="{% url donations_help username %}">Help</a></td>
				  {% endblock nav_help %}
				</tr>
			</table>
		</div>
	</div>
{% endblock navigation %}
