
/******************************
* STYLE FOR PROFILE PAGE
*******************************/

h1 { 
  font-size: 1.8em; 
  border-bottom: 1px solid #bbb; 
  width: 70%; 
  min-width: 688px;
}

#info {
  background-color : #ded;
  border : 1px solid #0a0;
  padding : 1em;
  margin-bottom : 1em;
}

#error_msg {
  background-color : #f2dbdb;
  border : 1px solid red;
  padding : 1em;
  margin-bottom : 1em;
}

ul.errorlist li {
	font-size: 12pt;
  padding: 0px;
}

/* ADDED July 27, 2012 to fix the table collapsing on itself when the browser is below a certain size */
table.profile {  
  border-collapse: collapse;
  border-spacing: 0;
  width: 70%;
  min-width: 703px; 
}

table.profile tr td.key {
  text-align: right;
  height: 30px;
	font-size: 16px;
  font-weight: bold;
  width: 160px;
}

table.profile tr td.value {
  padding-left: 20px;   
    min-width: 500px; /* ADDED July 27, 2012 */
}

table.profile button { 
  width: 80px;
  vertical-align: middle;
  float: right;
  margin-right: 15px;
}

table.profile span.hide_this{ 
	display: inline;
	display: none;
}

table.profile form{ 
	display:inline;
}

#download {
  float: right;
  border: 2px solid #666;
  background-color: #fff;
  padding: 10px;
  margin-right: 0px;
  text-align: center;
  font-weight: bold;
}

#download a img {
  display: block;
  width: 150px;
}

#download a {
  color: black;
}

#download_pubkey_form {
  display : inline;	
}
/* ADDED(everything below) Aug 14, 2012 */
#generate_api {
  width: 175px;
}

#change_key form {
  display: inline;
  width: 688px; 
}

#generate_keypair {
  position:relative;
  width: 175px;
  float: right;
  margin-right: 15px;

}
#edit_password table {
 display:inline;
 margin: auto;
  border-collapse: collapse;
  border-spacing: 0;
  padding-left: 5px;
  width: 100%;
}

#edit_password td.field {
  padding-left: 5px;

}

.wide_button {
  width: 175px;
}

.api_cell {
  color:red;
}

.api_cell:hover {
  text-decoration: underline; 
}

img.help {
  position: relative;
  float: left;
  left: 162px;
  display: none;
  /* when a negative margin is applied to right or bottom
  it allows the element to overlap
  */
  margin-bottom: -50px;
}

#id_email {
  width: 200px; 
}

#id_affiliation {
  width: 200px;
}

/*Added Sept 2012 to fix long api keys creating a new line*/
#api_key {
  color: red;
  font-family: courier;
}

#up_arrow {
  height: 36px;
  width: 36px;
  float: right;
  position: relative;
  margin-right: 132px;
  /*when a negative margin is applied to right or bottom
  it allows the element to overlap
  */
  margin-bottom:-50px;
}
