{% extends "accounts/accounts_base.html" %}

{% block title %}
Page not found - Seattle Clearinghouse
{% endblock title %}

{% block content %}
	<div id="main">
		<div id="middle">

			<h2>Hmm, the page you're looking for doesn't exist.</h2>
			
			<p>Our fault or yours, what do you think? If you think it's our fault, please contact us.</p>
			
		</div>
	</div>
{% endblock content %}
