{% extends "control/control_base.html" %}

{% block title %}
Get Donations - Seattle Clearinghouse
{% endblock title %}

{% block style %}
{{ block.super }}
<link href="{{ MEDIA_URL }}css/download.css" rel="stylesheet" type="text/css" />
{% endblock style %}

{% block script %}
{{ block.super }}
<script type="text/javascript" src="{{ MEDIA_URL }}scripts/jqbrowser.js" charset="utf-8"></script>
<script type="text/javascript" src="{{ MEDIA_URL }}scripts/download.js" charset="utf-8"></script>
{% endblock script %}

{% block nav_getdonations %}
<td class="active"><a href="{% url getdonations %}">Get Donations</a></td>
{% endblock nav_getdonations %}

{% block content %}
	<div id="main">
		<div id="middle">
			<div id="downloadlink">
				<p>Give this link to others so they can donate on your behalf:</p>
				<a href="{% url installers username %}">{{ domain }}{% url installers username %}</a>
			</div>
			{% include "common/installers.html" %}
		</div>
	</div>

{% endblock content %}
