			<h2><a name="contact">Who to contact for more help</a></h2>
			
			<p>The Seattle Clearinghouse portal is being actively worked
			on. We solicit and appreciate your comments
			concerning any aspect of your experience with
			using the web site and Seattle more
			generally.</p>

			<h3>Use our mailing list</h3>
			
			<p>Email Seattle users mailing list
			  at <a href="https://mailman.cs.washington.edu/mailman/listinfo/seattle-users">seattle-users@cs.washington.edu</a>
			  with any further questions about Seattle Clearinghouse,
			  Seattle or anything else. We love receiving
			  feedback and hearing from our users. If you
			  have a highly technical question, also email
			  our developers list
			  at <a href="https://mailman.cs.washington.edu/mailman/listinfo/seattle-devel">seattle-devel@cs.washington.edu</a>.</p>
			
			<h3>Read our Wiki</h3>
			
			<p>We have a growing collection of helpful
			  resources for users of Seattle, Seattle Clearinghouse, and Repy
			  (the Seattle programming language). Visit our
			  <a href="https://seattle.cs.washington.edu/">main
			    page</a> to get started, or jump to more
			  specific area of the site intended for
			  <a href="https://seattle.cs.washington.edu/wiki/ProgrammersPage">programmers</a>, 
			  <a href="https://seattle.cs.washington.edu/wiki/EducatorsPage">educators</a>,
			  and <a href="https://seattle.cs.washington.edu/wiki/ContributorsPage">contributors</a>.</p>
