from distutils.core import setup
setup(
  name = 'clearinghouse',
  packages = ['clearinghouse'], # this must be the same as the name above
  version = '0.1',
  description = "Seattle's seash package",
  author = 'Seattle',
  author_email = 'cc4256@nyu.edu',
  url = 'https://github.com/choksi81/clearinghouse', # use the URL to the github repo
  download_url = 'https://github.com/choksi81/clearinghouse/tarball/0.1', # I'll explain this in a second
  classifiers = [],
)
